/**
 * File:
 *  lcd.c
 *
 * Notes:
 *  This file contains the implementation of the LCD routines.
 *
 * History:
 *  02/06/2012 Created by Simon Platten
 */
#include "lcd.h"
// Lcd module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
/**
 * Function:
 *  initLCD
 *
 * Parameters:
 *  none
 *
 * Returns:
 *  none
 */
void initLCD() {
  Lcd_Init();
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);
}
/**
 * Function:
 *  lcdMaquee
 *
 * Parameters:
 *  pMarquee, ptr to marquee structure
 *
 * Returns:
 *  none
 *
 * Notes:
 *  This routine should be called from within a loop to iterate the marquee.
 */
void lcdMaquee(marqueeDef* pMarquee) {
  if ( pMarquee ) {
    char strLCD[LCD_CHAR_WIDTH + 1];
    int i, intLength, intCopyLen1, intCopyLen2, intSrc;
// How long is the marquee?
    intLength = strlen(pMarquee->pstrMarquee);
// Display an LCD display width of text
    intCopyLen1 = min(LCD_CHAR_WIDTH,  intLength - pMarquee->intStartIdx);

    if ( intCopyLen1 < LCD_CHAR_WIDTH ) {
      intCopyLen2 = LCD_CHAR_WIDTH - intCopyLen1;
    } else {
      intCopyLen2 = 0;
    }
    memset(strLCD, 0, sizeof(strLCD));
    intSrc = pMarquee->intStartIdx;
//    strncpy(strLCD, (char*)(pMarquee->pstrMarquee + intSrc), intCopyLen1);
    strncpy(strLCD, (char*)&pMarquee->pstrMarquee[intSrc], intCopyLen1);

    if ( intCopyLen2 > 0 ) {
       strncpy((char*)&strLCD[intCopyLen1], pMarquee->pstrMarquee, intCopyLen2);
    }
    lcdText(pMarquee->intRow, strLCD);
// Increment and catch wrap around
    if ( ++pMarquee->intStartIdx >= intLength ) {
      pMarquee->intStartIdx = 0;
    }
  }
}
/**
 * Function:
 *  lcdText
 *
 * Parameters:
 *  intOptions, use the following options:
 *                  LCD_ROW_1
 *                  LCD_ROW_2
 *                  LCD_CENTER
 *              The column may be or'd in with the options if required.
 *  pText, the text to display
 *
 * Returns:
 *  none
 */
void lcdText(int intOptions, char* pText) {
  int intLength, intCol, intRow;
// Is pText valid?
  if ( pText ) {
// Determine the length of the text
    for( intLength=0; pText[intLength]!='\0'; intLength++ ) {
    }
    if ( intLength > 0 ) {
// Determine the column
      if ( (intOptions & LCD_CENTER) == LCD_CENTER ) {
        intCol = ((LCD_CHAR_WIDTH - intLength) / 2) + 1;
      } else {
        intCol = intOptions & 0x0f;
        if ( intCol <= 0 ) {
          intCol = 1;
        } else if ( intCol >= LCD_CHAR_WIDTH ) {
          return;
        }
      }
      if ( (intOptions & LCD_ROW_1) == LCD_ROW_1 ) {
        intRow = 1;
      } else {
        intRow = 2;
      }
      Lcd_Out(intRow, intCol, pText);
    }
  }
}